"""
Analysis of FP data 

author : dorababu@subcom.tech
"""
import numpy as np
import serial
import time
import matplotlib.pyplot as plt
import fingerprint_enhancer 
import fingerprint_feature_extractor


from skimage.morphology import skeletonize, thin 

# replace space to new line
def create_np_array(file_name):
    '''
    Read data from text file and save in an array
    '''
    temps = []
    with open(file_name, "r") as r:
        with open("fpData/temp.txt", "w") as w:
            w.write(r.read().replace(' ' , '\n'))

    # Open text file and write to temp list
    f = open('fpData/temp.txt')
    for line in f.readlines():

        try:
            temps.append(int(line.strip()))
        except:
            print("Space")

    f.close()
    # store list in to numpy array
    fp_array = np.array(temps)
    return (fp_array)


# Display array
def display_fp(fp_arr):

    '''
    shape array to show as pixels 
    '''
    # Construct image from data  
    fp_image = np.reshape(fp_arr, (160,160))
    # print(test_arr)
    plt.imshow(fp_image, 'gray') 
    # plt.show()
    return (fp_image)

# enhance fingerprint image 
def enhance_fp_image(fp_img):
    '''
    enhance fingerprint
    '''
    out = fingerprint_enhancer.enhance_Fingerprint(fp_img)
    plt.imshow(out, 'gray')
    # plt.show()
    
    return (out)

# remove dot function for thinning fp image 

def removedot(invertThin):
    temp0 = np.array(invertThin[:])
    temp0 = np.array(temp0)
    temp1 = temp0/255
    temp2 = np.array(temp1)
    temp3 = np.array(temp2)

    enhanced_img = np.array(temp0)
    filter0 = np.zeros((10,10))
    W,H = temp0.shape[:2]
    filtersize = 6

    for i in range(W - filtersize):
        for j in range(H - filtersize):
            filter0 = temp1[i:i + filtersize,j:j + filtersize]

            flag = 0
            if sum(filter0[:,0]) == 0:
                flag +=1
            if sum(filter0[:,filtersize - 1]) == 0:
                flag +=1
            if sum(filter0[0,:]) == 0:
                flag +=1
            if sum(filter0[filtersize - 1,:]) == 0:
                flag +=1
            if flag > 3:
                temp2[i:i + filtersize, j:j + filtersize] = np.zeros((filtersize, filtersize))

    return temp2

# thinning fp image 
def thinning_fp_image(fp_img):
    '''
    thinning fp_image 
    '''
    # Normalize to 0 and 1 range 
    fp_img[fp_img == 255] = 1 

    skeleton = skeletonize(fp_img)
    skeleton = np.array(skeleton, dtype=np.uint8)
    skeleton = removedot(skeleton)

    # plt.imshow(skeleton, 'gray')
    # plt.show()

    # Saving the 2D array in test file 
    np.savetxt("fpData/skeleton.txt", skeleton)

    # display the contect of the text file 
    content = np.loadtxt("fpData/skeleton.txt")
    print("\nContent in skeleton.txt :\n", content)

    return(skeleton)

# fp feature extractor
def fp_feature_extract(fp_img):
    '''
    extract fingerprint features from fp data
    '''
    plt.imshow(fp_img, 'gray')
    plt.show()

    FeaturesTerminations, FeaturesBifurcations = fingerprint_feature_extractor.extract_minutiae_features(fp_img, spuriousMinutiaeThresh=10, invertImage = False, showResult=True, saveResult = True)

# main function
if __name__ == '__main__':

    fp_array1 = create_np_array("fpData/RawData1.txt")
    # fp_array2 = create_np_array("fpData/RawData6.txt")

    fp_image_array = display_fp(fp_array1)
    # display_fp(fp_array2)
    
    # Enhance fingerprint 
    fp_enhance_array = enhance_fp_image(fp_image_array)
    
    # Thinning image 
    fp_thinning_array = thinning_fp_image(fp_enhance_array)
    
    # extract features
    # fp_feature_extract(fp_thinning_array)

